'use strict'

const httpUtils = require('../http-api/http-utils')

const util = require('../util')
const Logger = util.getLogger(__dirname, __filename)

module.exports = init

function init (context) {
  return new SnapshotAction(context)
}

class SnapshotAction {
  constructor (context) {
    this._context = context
    // Keep track of when agents enter and leave this view and
    // if and when the last snapshot was triggered.
    this.type = 'snapshot'
  }

  run (params, cb) {
    // find action configuration
    params.actionConfig = this.getActionConfig(params)
    if (!params.actionConfig) return setImmediate(cb)

    Logger.info(`Starting snapshot on ${params.agent.id} for ${params.view.name || params.event}`)

    const assetCommand = {
      command: 'snapshot',
      id: params.agent.id,
      args: {
        metadata: {
          viewId: params.view.id,
          viewName: params.view.name,
          reason: 'View'
        }
      }
    }

    httpUtils.runAssetCommand(assetCommand, this._context, cb)
  }

  // Don't support more than one snapshot action at a time
  getActionConfig (params) {
    return params.view.actions.find(action => {
      return action && action.type === 'snapshot'
    })
  }
}
