'use strict'

exports.launch = launch

const util = require('../util')
const Logger = util.getLogger(__filename)
const SettingsManager = require('../settings-manager')

function launch (Context) {
  return new Promise((resolve, reject) => {
    const settingsManager = SettingsManager.create(Context)
    if (Context.config.data.postgresDsn) return resolve()

    settingsManager.readSettings((err, settings) => {
      if (err) return reject(err)

      Context.settingsManager = settingsManager

      if (!settings.generalSettings ||
          !settings.generalSettings.generalSettingsMain ||
          !settings.generalSettings.generalSettingsMain.didFirstSetup) {
        // First time setup
        settingsManager.setDefaultViews().then(
          result => { // Resolve
            Logger.info('Set preset views')

            const existingSettings = settings.generalSettings
              ? settings.generalSettings.generalSettingsMain
              : {}

            const settingsUpdate = Object.assign({}, existingSettings, {
              id: 'generalSettingsMain',
              didFirstSetup: true
            })
            settingsManager.setSetting('generalSettings', settingsUpdate, err => {
              if (err) {
                Logger.warn('Failed to complete first-time setup', err)
                return
              }
              Logger.info('Finished first-time setup')
            })
          },
          reason => { // Reject
            Logger.warn('Failed to set preset views', reason)
          }
        )
      }

      Logger.debug('Settings loaded.')
      resolve()
    })
  })
}
