'use strict'

exports.handleRequest = handleRequest

const util = require('../../util')
const Logger = util.getLogger(__dirname, __filename)

// Handle the HTTP request
function handleRequest (req, res) {
  const context = req.nsConsole.context
  const eventStream = req.nsConsole.eventStream
  const queryExpr = req.nsConsole.qData.q

  Logger.debug('handling request')

  context.settingsManager.streamAgents((err, record) => {
    if (err) {
      Logger.error('error reading info from settings')
      eventStream.close()
      return
    }

    if (record == null) {
      eventStream.close()
      return
    }

    const infoString = record.value
    const infoObject = JSON.parse(infoString)

    if (queryExpr != null) {
      if (!queryExpr.evalForAgent({info: infoObject})) return
    }

    const info = Object.assign({}, infoObject)
    info.time = new Date(info.processStart)
    eventStream.writeData(info)
  })
}
