'use strict'

exports.handleRequest = handleRequest

const util = require('../../util')
const Logger = util.getLogger(__dirname, __filename)

const API = require('./API')
const VERSION = API.VERSION
const HTTP_VERBS = ['GET', 'PUT', 'POST', 'DELETE']

// Handle the HTTP request
function handleRequest (req, res) {
  const context = req.nsConsole.context

  Logger.debug('handling request')

  res.status(200)
  res.send(pivotSchema(context.schemas.http[VERSION]))
}

let cached
function pivotSchema (schema) {
  if (cached != null) {
    return cached
  }
  /* pivot by HTTP verb to return ~
    {
      "command": {
        "GET": {
          "params": ["q"],
          "required": ["q"],
          "response": "blah",
          "keepalive": false
        },
        "PUT": {
          "params": ["q"],
          "required": ["q"],
          "response": "blah",
          "keepalive": false
        },
      }
    }
  */

  const pivoted = {}

  for (let key in schema) {
    if (HTTP_VERBS.indexOf(key) < 0) {
      continue
    }
    for (let command in schema[key]) {
      if (pivoted[command] == null) {
        pivoted[command] = {}
      }
      const defn = schema[key][command]

      pivoted[command][key] = {
        params: (defn.parms != null && defn.parms.length > 0) ? defn.parms : [],
        required: (defn.required != null && defn.required.length > 0) ? defn.required : [],
        response: defn.response || 'json',
        keepalive: defn.keepalive || false,
        request: defn.request
      }
    }
  }
  cached = pivoted
  return pivoted
}
