'use strict'

exports.handleRequest = handleRequest

const util = require('../../util')
const Logger = util.getLogger(__dirname, __filename)
const httpUtils = require('../http-utils')

// Handle the HTTP request
function handleRequest (req, res) {
  const context = req.nsConsole.context
  const qData = req.nsConsole.qData

  const diskAssetManager = context.diskAssetManager
  if (diskAssetManager == null) {
    res.status(501)
    Logger.error('diskAssetManager not initialized')
    return res.send({ error: 'Server error' })
  }

  const asset = diskAssetManager.byId(qData.id)
  const summaryAsset = diskAssetManager.summaryAsset(qData.id)

  if (!asset) {
    res.status(404)
    return res.send({ error: 'Asset not found' })
  }

  const parms = req.body

  const metadata = {}

  if (parms.starred != null) {
    const starred = parms.starred
    if (starred !== true && starred !== false) {
      res.status(400)
      res.send({ error: `Invalid value for starred: ${starred}` })
    }
    if (asset.starred !== starred) {
      metadata.starred = starred
    }
  }

  if (parms.visualization != null) {
    const visualization = parms.visualization
    if (visualization !== 'flamegraph' && visualization !== 'sunburst' && visualization !== 'treemap') {
      res.status(400)
      res.send({ error: `Invalid value for visualization: ${visualization}` })
    }

    if (asset.visualization !== visualization) {
      metadata.visualization = visualization
    }
  }
  metadata.sourceMap = parms.sourceMap != null ? !!parms.sourceMap : false

  // If anything changed, perform the updates
  if (Object.keys(metadata).length) {
    diskAssetManager.update(qData.id, metadata)

    // If we are updating a summary, also update the real thing
    if (asset.snapshotAsset) {
      const parentAsset = diskAssetManager.byId(asset.snapshotAsset)
      if (parentAsset) {
        diskAssetManager.update(asset.snapshotAsset, metadata)
      }
    }

    // and vice versa
    if (summaryAsset) {
      diskAssetManager.update(summaryAsset.asset, metadata)
    }
  }

  res.status(200)
  res.send(httpUtils.normalizeAsset(asset))
}
