'use strict'

exports.handleRequest = handleRequest

const util = require('../../util')
const Logger = util.getLogger(__dirname, __filename)

// Handle the HTTP request
function handleRequest (req, res) {
  const context = req.nsConsole.context
  const qData = req.nsConsole.qData

  const diskAssetManager = context.diskAssetManager
  if (diskAssetManager == null) {
    res.status(501)
    Logger.error('diskAssetManager not initialized')
    return res.send({ error: 'Server error' })
  }

  const asset = diskAssetManager.byId(qData.id)
  const summaryAsset = diskAssetManager.summaryAsset(qData.id)

  if (!asset) {
    res.status(404)
    return res.send({ error: 'Asset not found' })
  }

  diskAssetManager.restore(qData.id)
  // If restoring main asset and have a summary, restore that too
  if (summaryAsset) {
    diskAssetManager.restore(summaryAsset.asset)
  }
  // If restore summary, restore main asset
  if (asset.snapshotAsset) {
    diskAssetManager.restore(asset.snapshotAsset)
  }

  res.status(200)
  res.send(asset)
}
