'use strict'

exports.handleRequest = handleRequest

// Handle the HTTP request
function handleRequest (req, res) {
  const context = req.nsConsole.context
  const qData = req.nsConsole.qData

  const vulnManager = context.vulnManager

  // get the request body
  const parms = req.body
  if (parms == null) {
    res.status(400)
    return res.send({ error: 'request body expected' })
  }

  // only checking the `hidden` property for now
  if (typeof parms.hidden !== 'boolean') {
    res.status(400)
    return res.send({ error: 'request body expected to have a boolean property "hidden"' })
  }

  // get the vuln
  const vuln = vulnManager.getVulnById(qData.id)
  if (vuln == null) {
    res.status(404)
    return res.send({ error: `vulnerability ${qData.id} not found` })
  }

  // update the hidden vulns
  vulnManager.setHiddenVulnId(vuln.id, parms.hidden, (err) => {
    if (err) {
      res.status(500)
      return res.send({ error: `error setting vuln metadata: ${err}` })
    }

    // get the updated vuln, should have updated metadata
    const updatedVuln = vulnManager.getVulnById(vuln.id)
    if (updatedVuln == null) {
      res.status(500)
      return res.send({ error: `error setting vuln metadata: vuln not found` })
    }

    // return the metadata
    res.status(200)
    res.send(updatedVuln.nsolidMetaData)
  })
}
