'use strict'

// const lodash = require('lodash')

const api = require('./api')
// const util = require('../util')

exports.create = create

// const Logger = util.getLogger(__dirname, __filename)

// Create and return a new database writer.
function create (context) {
  return new Reader(context)
}

class Reader {
  constructor (context) {
    this.context = context
    this._user = context.config.influxdb.user !== '' ? context.config.influxdb.user : null
    this._password = context.config.influxdb.password !== '' ? context.config.influxdb.password : null
    this._errorState = false
  }

  // Read a metrics record from the database.
  readMetrics (query, cb) {
    // If influxdb isn't fully launched, return the empty set, which is a
    // JSON-encoded empty array because api.query doesn't decode
    if (!this.context.influxdbProcess && !this.context.usingExternalInflux) {
      return cb(null, '[]')
    }

    this._read(query, cb)
  }

  // Read events records from the database.
  readEvents (query, cb) {
    // If influxdb isn't fully launched, return the empty set, which is a
    // JSON-encoded empty array because api.query doesn't decode
    if (!this.context.influxdbProcess && !this.context.usingExternalInflux) {
      return cb(null, '[]')
    }

    this._read(query, cb)
  }

  // Read data with a query.
  _read (query, cb) {
    api.query(this.context.config.influxdb.url, 'nsolid', query, this._user, this._password, (err, data) => {
      if (err) {
        if (this.context.eventManager) {
          this.context.eventManager.emitSystemEvent('influxdb-error', null, { message: 'read error' })
        }
        this._errorState = true
        return cb(err)
      }

      if (this._errorState) {
        if (this.context.eventManager) {
          this.context.eventManager.emitSystemEvent('influxdb-recovered', null, { message: 'read recovery' })
        }
        this._errorState = false
      }

      cb(null, data)
    })
  }
}
