const multer = require('multer')
const { diskStorage } = multer

function storageStrategy (sourcemapDirectoryResolver) {
  async function determineDirectory (req, file, cb) {
    try {
      const assetDir = await sourcemapDirectoryResolver.fromRequest(req)
      cb(null, assetDir)
    } catch (err) {
      cb(err)
    }
  }

  function determineFileName (req, file, cb) {
    cb(null, file.originalname)
  }

  return {
    determineDirectory,
    determineFileName
  }
}

module.exports = function multerStorage (sourcemapDirectoryResolver) {
  const strategy = storageStrategy(sourcemapDirectoryResolver)
  return diskStorage({
    destination: strategy.determineDirectory,
    filename: strategy.determineFileName
  })
}
