'use strict'

const path = require('path')

class AssetNotFoundError extends Error {
  constructor (msg) {
    super(msg)
    this.name = AssetNotFoundError.Name
  }

  static get Name () { return 'AssetNotFoundError' }
}

class SourcemapDirectoryResolver {
  constructor (diskAssetManager) {
    this._diskAssetManager = diskAssetManager
  }

  async fromRequest (req) {
    // We are storing the sourcemap file in the profiles folder,
    // right next to the profile
    // In the future we might get smarter and detect that a profile
    // with the same "main" can use the same uploaded sourcemaps.
    // However since this is a band aid we'll keep things simple.
    const assetID = req.query.assetID
    const metadata = this._diskAssetManager.byId(assetID)
    if (metadata == null) throw new AssetNotFoundError(`Asset ${assetID} not found`)

    const fullAssetProfilePath = path.join(
      this._diskAssetManager.assetsDir,
      metadata.fileName
    )
    return path.dirname(fullAssetProfilePath)
  }
}

module.exports = { SourcemapDirectoryResolver, AssetNotFoundError }
