'use strict'

const { ensureDirExists } = require('../../util')

const { SourcemapDirectoryResolver } =
  require('./sourcemap-directory-resolver')

const multer = require('multer')
const multerStorage = require('./multer-storage')

function ensureParams (req, res, next) {
  const { assetID } = req.query
  if (assetID == null) {
    res
      .status(422)
      .json({ result: null, error: 'Missing assetID in body' })
    return
  }
  next()
}

function ensureSourcemapDirectory (sourcemapDirectoryResolver) {
  return async function ensureDirectory (req, res, next) {
    try {
      const dir = await sourcemapDirectoryResolver.fromRequest(req)
      ensureDirExists(dir)
      next()
    } catch (err) {
      next(err)
    }
  }
}

module.exports = function initSourcemapUploadRoute (diskAssetManager) {
  const sourcemapDirectoryResolver = new SourcemapDirectoryResolver(diskAssetManager)
  const storage = multerStorage(sourcemapDirectoryResolver)

  async function onUploadCompleted (req, res, next) {
    const { file } = req
    try {
      if (file == null) {
        throw new Error('Uploaded sourcemap not properly stored')
      }
      const assetID = req.query.assetID
      const metadata = diskAssetManager.byId(assetID)
      metadata.sourcemapZipFile = file.path
      diskAssetManager.update(assetID, metadata)

      res.sendStatus(200)
    } catch (err) {
      next(err)
    }
  }

  return [
    ensureParams,
    ensureSourcemapDirectory(sourcemapDirectoryResolver),
    multer({ storage }).single('sourcemap'),
    onUploadCompleted
  ]
}
