'use strict'

const profileResolver = require('@ns-private/sm-profile-resolver')
const { ProjectInaccessibleError } = profileResolver

async function tryResolveSourcemap (res, resolver, fileUrl) {
  try {
    const sourcemap = await resolver.resolveSourcemap(fileUrl)
    res
      .status(200)
      .json({
        result: sourcemap && sourcemap.sourcemap && sourcemap.sourcemap.sourcemapData,
        error: null
      })
  } catch (err) {
    if (err.name === ProjectInaccessibleError.Name) {
      res
        .status(404)
        .json({ result: null, error: err.message, projectInaccessible: true })
    } else {
      res
        .status(500)
        .json({ result: null, error: err.message })
    }
  }
}

module.exports = function initSourcemapRoute () {
  const adjacentMapFileResolver = new profileResolver.AdjacentMapFileResolver(true)
  return async function sourcemap (req, res) {
    const { fileUrl, zipMapFile } = req.body
    if (fileUrl == null) {
      res
        .status(422)
        .json({ result: null, error: 'Missing fileUrl in body' })
      return
    }

    if (zipMapFile == null) {
      return tryResolveSourcemap(res, adjacentMapFileResolver, fileUrl)
    } else {
      const resolver = profileResolver.ZipMapFileResolver
        .cachedInstance(zipMapFile)
      return tryResolveSourcemap(res, resolver, fileUrl)
    }
  }
}
