'use strict'

const cloneDeep = require('lodash/cloneDeep')
const partition = require('deep-partition-layout')

const initializeFlamegraph = (head, measureTooltipLabel, widthCanvas = 1, heightCanvas = 1, offsetX = 0, offsetY = 0) => {
  let rawProfile = cloneDeep(head)

  return partition()
    .size([widthCanvas, heightCanvas])
    .children(d => d.children)
    .value(d => d.hitCount)
    .sort(null)
    .nodes(rawProfile)
    .map(rawNode => {
      const functionName = rawNode.functionName || '(anonymous)'

      const width = rawNode.dx
      const height = rawNode.dy

      return {
        rect: {
          x: rawNode.x + offsetX,
          y: (heightCanvas - rawNode.y) - height + offsetY,
          width,
          height
        },
        id: rawNode.id,
        functionName,
        url: rawNode.url,
        codeType: rawNode.codeType,
        lineNumber: rawNode.lineNumber,
        totalTime: rawNode.totalTime.toFixed(2),
        labelMeasurements: measureTooltipLabel(functionName)
      }
    })
}

exports.initializeFlamegraph = initializeFlamegraph

const baseColors = ['#4a5655', '#647572', '#343d3c']

exports.baseColors = baseColors
