'use strict'

const fs = require('fs')
const tape = require('tape')
const locateNode = require('../src/locate-node')
const extractOutline = require('../src/extract-outline')
const filterData = require('../src/filter-data')

const smallParsedAsset = JSON.parse(fs.readFileSync('./test/fixtures/small.cpuprofile'))

tape('locateNode', t => {
  extractOutline(smallParsedAsset)
  const filteredAsset = filterData(smallParsedAsset, {
    showNSolid: false,
    showProgram: false,
    showGarbage: false,
    showIdle: false
  })

  const result = locateNode(filteredAsset.head, 3)

  t.equals(result.length, 2, 'rows returned')

  const startProfiling = result[0]
  t.equals(startProfiling.functionName, 'startProfiling')
  t.equals(startProfiling.url, 'profiler.js')
  t.equals(startProfiling.lineNumber, 192)
  t.equals(startProfiling.callUID, 2)
  t.equals(startProfiling.bailoutReason, 'no reason')
  t.equals(startProfiling.id, 3)
  t.equals(startProfiling.scriptId, 82)
  t.equals(startProfiling.hitCount, 0)
  t.equals(startProfiling.children.length, 1)
  t.equals(startProfiling.selfTime, 0)
  t.equals(startProfiling.totalHitCount, 1)
  t.equals(startProfiling.totalTime, 1.814)
  t.equals(startProfiling.short, 'profiler.js')
  t.equals(startProfiling.codeType, 'core')

  const root = result[1]
  t.equals(root.functionName, '(root)')
  t.equals(root.url, '')
  t.equals(root.lineNumber, 0)
  t.equals(root.callUID, 10)
  t.equals(root.bailoutReason, '')
  t.equals(root.id, 1)
  t.equals(root.scriptId, 0)
  t.equals(root.hitCount, 0)
  t.equals(root.children.length, 2)
  t.equals(root.selfTime, 0)
  t.equals(root.totalHitCount, 2)
  t.equals(root.totalTime, 4.769)
  t.equals(root.short, '.')
  t.equals(root.codeType, 'core')

  t.end()
})
