'use strict'

const fs = require('fs')
const tape = require('tape')
const almostEqual = require('almost-equal')
const { extractOutline, filterData } = require('../index')
const { sumOutline } = require('./util')
const largeParsedAsset = JSON.parse(fs.readFileSync('./test/fixtures/small.cpuprofile'))

tape('check a few nodes in a small profile', (t) => {
  const extractedOutline = extractOutline(largeParsedAsset)
  const filteredAsset = filterData(largeParsedAsset, {
    showNSolid: false,
    showProgram: false,
    showGarbage: false,
    showIdle: false
  })

  const expectedOutline = {
    application: 0.00031853896084161617,
    gc: 0,
    idle: 0.9995984365207423,
    program: 0.00008302451841604716
  }
  t.ok(almostEqual(sumOutline(extractedOutline), 1, almostEqual.FLT_EPSILON, almostEqual.FLT_EPSILON), 'outline adds up to 1.0')
  t.deepEquals(extractedOutline, expectedOutline)

  t.equals(filteredAsset.typeId, 'CPU')
  t.equals(filteredAsset.uid, '1')
  t.equals(filteredAsset.title, 'f44fd3bd-5e35-40fd-bb40-64218e5505da')
  t.equals(filteredAsset.startTime, 99847)
  t.equals(filteredAsset.endTime, 99862)
  t.equals(filteredAsset.samples.length, 12325)
  t.equals(filteredAsset.timestamps.length, 12325)
  t.equals(Object.keys(filteredAsset.UIDs).length, 5)

  const root = filteredAsset.head
  t.equals(root.functionName, '(root)')
  t.equals(root.url, '')
  t.equals(root.lineNumber, 0)
  t.equals(root.bailoutReason, '')
  t.equals(root.id, 1)
  t.equals(root.scriptId, 0)
  t.equals(root.hitCount, 0)
  t.equals(root.children.length, 2)

  const startProfiling = root.children[0]
  t.equals(startProfiling.functionName, 'startProfiling')
  t.equals(startProfiling.url, 'profiler.js')
  t.equals(startProfiling.lineNumber, 192)
  t.equals(startProfiling.callUID, 2)
  t.equals(startProfiling.bailoutReason, 'no reason')
  t.equals(startProfiling.id, 3)
  t.equals(startProfiling.scriptId, 82)
  t.equals(startProfiling.hitCount, 0)
  t.equals(startProfiling.children.length, 1)
  t.equals(startProfiling.lineTicks, undefined)
  t.equals(startProfiling.selfTime, 0)
  t.equals(startProfiling.parent, root)
  t.equals(startProfiling.totalHitCount, 1)
  t.equals(startProfiling.totalTime, 1.814)
  t.equals(startProfiling.short, 'profiler.js')
  t.equals(startProfiling.codeType, 'core')

  const memoryUsage = root.children[1]
  t.equals(memoryUsage.functionName, 'memoryUsage')
  t.equals(memoryUsage.url, 'internal/process.js')
  t.equals(memoryUsage.lineNumber, 102)
  t.equals(memoryUsage.callUID, 7)
  t.equals(memoryUsage.bailoutReason, 'no reason')
  t.equals(memoryUsage.id, 9)
  t.equals(memoryUsage.scriptId, 26)
  t.equals(memoryUsage.hitCount, 0)
  t.equals(memoryUsage.children.length, 1)
  t.equals(memoryUsage.lineTicks, undefined)
  t.equals(memoryUsage.selfTime, 0)
  t.equals(memoryUsage.parent, root)
  t.equals(memoryUsage.totalHitCount, 1)
  t.equals(memoryUsage.totalTime, 2.955)
  t.equals(memoryUsage.short, 'internal/process.js')
  t.equals(memoryUsage.codeType, 'core')

  const memoryUsageToo = memoryUsage.children[0]
  t.equals(memoryUsageToo.functionName, 'memoryUsage')
  t.equals(memoryUsageToo.url, '')
  t.equals(memoryUsageToo.lineNumber, 0)
  t.equals(memoryUsageToo.callUID, 6)
  t.equals(memoryUsageToo.bailoutReason, '')
  t.equals(memoryUsageToo.id, 10)
  t.equals(memoryUsageToo.scriptId, 0)
  t.equals(memoryUsageToo.hitCount, 1)
  t.equals(memoryUsageToo.children.length, 0)
  t.equals(memoryUsageToo.lineTicks.length, 1)
  t.equals(memoryUsageToo.selfTime, 2.955)
  t.equals(memoryUsageToo.parent, memoryUsage)
  t.equals(memoryUsageToo.totalHitCount, 1)
  t.equals(memoryUsageToo.totalTime, 2.955)
  t.equals(memoryUsageToo.short, '.')
  t.equals(memoryUsageToo.codeType, 'core')

  const setSamplingInterval = startProfiling.children[0]
  t.equals(setSamplingInterval.functionName, 'setSamplingInterval')
  t.equals(setSamplingInterval.url, '')
  t.equals(setSamplingInterval.lineNumber, 0)
  t.equals(setSamplingInterval.callUID, 1)
  t.equals(setSamplingInterval.bailoutReason, '')
  t.equals(setSamplingInterval.id, 4)
  t.equals(setSamplingInterval.scriptId, 0)
  t.equals(setSamplingInterval.hitCount, 1)
  t.equals(setSamplingInterval.children.length, 0)
  t.deepEquals(setSamplingInterval.lineTicks, [ { hitCount: 1, line: 205 } ])
  t.equals(setSamplingInterval.selfTime, 1.814)
  t.equals(setSamplingInterval.parent, startProfiling)
  t.equals(setSamplingInterval.totalHitCount, 1)
  t.equals(setSamplingInterval.totalTime, 1.814)
  t.equals(setSamplingInterval.short, '.')
  t.equals(setSamplingInterval.codeType, 'core')

  t.end()
})
