"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const fs_utils_1 = require("./fs-utils");
const path_utils_1 = require("./path-utils");
const Sourcemap_1 = require("./Sourcemap");
const SourcemapResolver_1 = require("./SourcemapResolver");
class ProjectInaccessibleError extends Error {
    constructor(msg) {
        super(msg);
        this.name = ProjectInaccessibleError.Name;
    }
    static get Name() { return 'ProjectInaccessibleError'; }
}
exports.ProjectInaccessibleError = ProjectInaccessibleError;
async function canAccessProject(url) {
    const canAccessSourceFile = await fs_utils_1.canRead(url);
    if (canAccessSourceFile)
        return true;
    const directory = path_1.dirname(url);
    return fs_utils_1.isAccessibleDirectory(directory);
}
class AdjacentMapFileResolver extends SourcemapResolver_1.SourcemapResolver {
    constructor(verifyProjectAccess = false) {
        super();
        this.verifyProjectAccess = verifyProjectAccess;
    }
    async resolveSourcemap(url) {
        if (!path_utils_1.isuserFile(url))
            return SourcemapResolver_1.ResolveSourcemapResult.empty;
        if (this.verifyProjectAccess) {
            const haveAccess = await canAccessProject(url);
            if (!haveAccess) {
                throw new ProjectInaccessibleError(`Cannot access project for ${url}.`);
            }
        }
        const json = await this._readMapFile(url);
        if (json == null)
            return SourcemapResolver_1.ResolveSourcemapResult.empty;
        return new SourcemapResolver_1.ResolveSourcemapResult(Sourcemap_1.Sourcemap.fromRawSourcemapJSON(json));
    }
    _mapFilePath(generatedFile) {
        return `${generatedFile}.map`;
    }
    async _readMapFile(generatedFile) {
        const p = this._mapFilePath(generatedFile);
        const readable = await fs_utils_1.canRead(p);
        if (!readable)
            return null;
        return fs_utils_1.readFile(p, 'utf8');
    }
}
exports.AdjacentMapFileResolver = AdjacentMapFileResolver;
//# sourceMappingURL=AdjacentMapFileResolver.js.map