"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cross_fetch_1 = __importDefault(require("cross-fetch"));
const path_utils_1 = require("./path-utils");
const Sourcemap_1 = require("./Sourcemap");
const SourcemapResolver_1 = require("./SourcemapResolver");
async function projectAccessible(res) {
    try {
        const body = await res.json();
        return !body.projectInaccessible;
    }
    catch (err) {
        return false;
    }
}
exports.defaultGenerateRequestInfo = (fileUrl, zipMapFile) => {
    const payload = zipMapFile != null ? { fileUrl, zipMapFile } : { fileUrl };
    return {
        method: 'POST',
        credentials: 'include',
        mode: 'cors',
        headers: {
            'Content-Type': 'application/json',
            'Accept': 'application/json'
        },
        body: JSON.stringify(payload)
    };
};
class RemoteFileResolver extends SourcemapResolver_1.SourcemapResolver {
    constructor(_endpoint, _zipMapFile = null, _generateRequestInfo = exports.defaultGenerateRequestInfo, cache = true) {
        super(cache);
        this._endpoint = _endpoint;
        this._zipMapFile = _zipMapFile;
        this._generateRequestInfo = _generateRequestInfo;
    }
    async resolveSourcemap(fileUrl) {
        if (!path_utils_1.isuserFile(fileUrl))
            return SourcemapResolver_1.ResolveSourcemapResult.empty;
        const requestInfo = this._generateRequestInfo(fileUrl, this._zipMapFile);
        try {
            const res = await cross_fetch_1.default(this._endpoint, requestInfo);
            if (res.status === 404) {
                return await projectAccessible(res)
                    ? SourcemapResolver_1.ResolveSourcemapResult.empty
                    : SourcemapResolver_1.ResolveSourcemapResult.failureResult(0 /* projectInaccessible */, `Cannot access the project containing ${fileUrl}`);
            }
            if (!res.ok)
                return SourcemapResolver_1.ResolveSourcemapResult.empty;
            const body = await res.json();
            if (body.error != null)
                throw new Error(body.error);
            if (body.result == null)
                return SourcemapResolver_1.ResolveSourcemapResult.empty;
            return new SourcemapResolver_1.ResolveSourcemapResult(Sourcemap_1.Sourcemap.fromRawSourcemap(body.result));
        }
        catch (err) {
            return SourcemapResolver_1.ResolveSourcemapResult.failureResult(1 /* unknown */, err.toString());
        }
    }
}
exports.RemoteFileResolver = RemoteFileResolver;
//# sourceMappingURL=RemoteFileResolver.js.map