"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const source_map_1 = require("source-map");
const SourceInfo_1 = require("./SourceInfo");
class SourcemapData {
    constructor(v3map, includeContents = false) {
        // not interested in names
        this.names = [];
        const { version, file, sourceRoot, sources, mappings } = v3map;
        this.version = version;
        this.file = file;
        this.sourceRoot = sourceRoot || '';
        this.sources = sources;
        this.mappings = mappings;
        if (includeContents)
            this.sourcesContent = v3map.sourcesContent;
    }
}
exports.SourcemapData = SourcemapData;
class Sourcemap {
    static fromRawSourcemapJSON(json) {
        const map = JSON.parse(json);
        return new Sourcemap(map);
    }
    static fromRawSourcemap(map) {
        return new Sourcemap(map);
    }
    constructor(map, includeContents = false) {
        this._sourcemapData = new SourcemapData(map, includeContents);
    }
    get sourcemapData() { return this._sourcemapData; }
    async originalPosition(sourceInfo) {
        if (this._consumer == null) {
            this._consumer = await new source_map_1.SourceMapConsumer(this._sourcemapData);
        }
        const mappedPosition = this._consumer.originalPositionFor(sourceInfo);
        const originalFile = this._sourcemapData.sources.length > 0
            ? this._sourcemapData.sources[0]
            : sourceInfo.url;
        return SourceInfo_1.SourceInfo.fromPosition(originalFile, mappedPosition);
    }
}
exports.Sourcemap = Sourcemap;
//# sourceMappingURL=Sourcemap.js.map