"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const jszip_1 = require("jszip");
const assert = require("assert");
const fs_utils_1 = require("./fs-utils");
const Sourcemap_1 = require("./Sourcemap");
const SourcemapResolver_1 = require("./SourcemapResolver");
class ZipMapFileResolver extends SourcemapResolver_1.SourcemapResolver {
    constructor(_zipFilePath, _mapFileFromPath = ZipMapFileResolver.adjacentMapFileFromPath) {
        super();
        this._zipFilePath = _zipFilePath;
        this._mapFileFromPath = _mapFileFromPath;
    }
    static cachedInstance(zipFilePath, mapFileFromPath) {
        if (ZipMapFileResolver._resolverCache.has(zipFilePath)) {
            return ZipMapFileResolver._resolverCache.get(zipFilePath);
        }
        const resolver = new ZipMapFileResolver(zipFilePath, mapFileFromPath);
        ZipMapFileResolver._resolverCache.set(zipFilePath, resolver);
        return resolver;
    }
    static removeFromCache(zipFilePath) {
        if (!ZipMapFileResolver._resolverCache.has(zipFilePath))
            return;
        ZipMapFileResolver._resolverCache.delete(zipFilePath);
    }
    async resolveSourcemap(url) {
        if (this._loadedZip == null)
            await this._loadZip();
        assert(this._loadedZip != null, 'Did not load zip correctly');
        const matchingFile = this._matchUrl(url);
        if (matchingFile == null)
            return SourcemapResolver_1.ResolveSourcemapResult.empty;
        const json = await matchingFile.async('text');
        return new SourcemapResolver_1.ResolveSourcemapResult(Sourcemap_1.Sourcemap.fromRawSourcemapJSON(json));
    }
    // @protected for testing only
    async _loadZip() {
        const src = await fs_utils_1.readFile(this._zipFilePath);
        this._loadedZip = await jszip_1.loadAsync(src);
    }
    // @protected for testing only
    _matchUrl(url) {
        const matches = this._loadedZip.filter((_relDir, file) => {
            if (file.dir)
                return false;
            return url.includes(file.name);
        });
        if (matches == null || matches.length === 0)
            return null;
        const generatedFile = matches[0];
        const mapFilePath = this._mapFileFromPath(generatedFile.name);
        return this._loadedZip.file(mapFilePath);
    }
}
ZipMapFileResolver.adjacentMapFileFromPath = path => `${path}.map`;
ZipMapFileResolver._resolverCache = new Map();
exports.ZipMapFileResolver = ZipMapFileResolver;
//# sourceMappingURL=ZipMapFileResolver.js.map