import fs from 'fs'
import {
  CpuProfile,
  CpuProfileNode,
  SourceInfoMapper
} from 'sm-profile-resolver'

const profileJSONPath =
  `${__dirname}/../../typescript/profiles/mapfile.json`
const profileJSON = JSON.parse(fs.readFileSync(profileJSONPath, 'utf8'))
const profile = profileJSON as unknown as CpuProfile
const nodes = profile.head.children
const endpoint = 'http://localhost:9002/sourcemap'
const mapper = SourceInfoMapper.forRemoteFile(endpoint)

function dumpUrls(mappedNodes: CpuProfileNode[], search: string) {
  for (const node of mappedNodes) {
    if (node.url != null && node.url.includes(search)) {
      console.log(node.url)
    }
    if (node.children != null && node.children.length > 0) {
      dumpUrls(node.children, search)
    }
  }
}
async function run() {
  try {
    const errors = await mapper.map(nodes)
    if (errors.length > 0) console.error(errors)

    const mappedJSON = JSON.stringify(nodes, null, 2)
    dumpUrls(nodes, 'cpu-')

    fs.writeFileSync(
      `${__dirname}/../../typescript/profiles/mapfile.mapped.json`,
      mappedJSON
    )
  } catch (err) {
    console.error(err)
  }
}

run()