import { NullableMappedPosition, Position } from 'source-map'
import { CpuProfileNode } from './CpuProfile'

export class SourceInfo implements Position {
  static fromNode(node: CpuProfileNode): SourceInfo {
    return new SourceInfo(node.url, node.functionName || null, node.lineNumber)
  }

  static fromPosition(url: string, pos: NullableMappedPosition): SourceInfo {
    return new SourceInfo(url, pos.name || null, pos.line || 0, pos.column || 0)
  }

  constructor(
    public url: string,
    public functionName: string | null,
    public line: number,
    public column = 0
  ) { }

  clone() {
    return new SourceInfo(
      this.url,
      this.functionName,
      this.line,
      this.column)
  }

  withColumn(column: number) {
    const cloned = this.clone()
    cloned.column = column
    return cloned
  }
}