import {
  NullableMappedPosition,
  RawSourceMap,
  SourceMapConsumer
} from 'source-map'

import { SourceInfo } from './SourceInfo'

export class SourcemapData implements RawSourceMap {
  version: number
  // not interested in names
  names: string[] = []
  file: string
  sourceRoot: string
  sources: string[]
  mappings: string
  sourcesContent?: string[]

  constructor(v3map: RawSourceMap, includeContents = false) {
    const { version, file, sourceRoot, sources, mappings } = v3map
    this.version = version
    this.file = file
    this.sourceRoot = sourceRoot || ''
    this.sources = sources
    this.mappings = mappings
    if (includeContents) this.sourcesContent = v3map.sourcesContent
  }
}

export class Sourcemap {
  static fromRawSourcemapJSON(json: string): Sourcemap {
    const map = JSON.parse(json) as RawSourceMap
    return new Sourcemap(map)
  }

  static fromRawSourcemap(map: RawSourceMap): Sourcemap {
    return new Sourcemap(map)
  }

  private readonly _sourcemapData: SourcemapData
  private _consumer?: SourceMapConsumer

  constructor(map: RawSourceMap, includeContents = false) {
    this._sourcemapData = new SourcemapData(map, includeContents)
  }

  get sourcemapData() { return this._sourcemapData }

  async originalPosition(sourceInfo: SourceInfo): Promise<SourceInfo> {
    if (this._consumer == null) {
      this._consumer = await new SourceMapConsumer(this._sourcemapData)
    }
    const mappedPosition: NullableMappedPosition =
      this._consumer.originalPositionFor(sourceInfo)

    const originalFile = this._sourcemapData.sources.length > 0
      ? this._sourcemapData.sources[0]
      : sourceInfo.url
    return SourceInfo.fromPosition(originalFile, mappedPosition)
  }
}
