import { Sourcemap } from './Sourcemap'

export const enum ResolveSourcemapFailureReason { projectInaccessible, unknown }

export class ResolveSourcemapResult {

  static get empty(): ResolveSourcemapResult {
    return new ResolveSourcemapResult(null)
  }

  get hasError(): boolean { return this.failureReason != null }
  get isEmpty(): boolean { return this.sourcemap == null }

  static failureResult(
    reason: ResolveSourcemapFailureReason,
    errorMessage: string
  ) {
    return new ResolveSourcemapResult(null, reason, errorMessage)
  }
  constructor(
    public sourcemap: Sourcemap | null,
    public failureReason: ResolveSourcemapFailureReason | null = null,
    public errorMessage: string | null = null
  ) { }
}

export abstract class SourcemapResolver {
  constructor(protected _cache = true) { }

  abstract async resolveSourcemap(url: string): Promise<ResolveSourcemapResult>
}
