import fs from 'fs'
import { CpuProfile, CpuProfileNode } from './CpuProfile'
import { SourceInfoMapper } from './SourceInfoMapper'

const profileJSONPath =
  `${process.cwd()}/samples/typescript/profiles/mapfile.json`
const profileJSON = JSON.parse(fs.readFileSync(profileJSONPath, 'utf8'))
const profile = profileJSON as unknown as CpuProfile
const nodes = profile.head.children
const mapper = SourceInfoMapper.forAdjacentFile()

function dumpUrls(mappedNodes: CpuProfileNode[], search: string) {
  for (const node of mappedNodes) {
    if (node.url != null && node.url.includes(search)) {
      console.log(node.url)
    }
    if (node.children != null && node.children.length > 0) {
      dumpUrls(node.children, search)
    }
  }
}
async function run() {
  try {
    await mapper.map(nodes)
    const mappedJSON = JSON.stringify(nodes, null, 2)
    dumpUrls(nodes, 'cpu-')

    fs.writeFileSync(
      `${__dirname}/../samples/typescript/profiles/mapfile.mapped.json`,
      mappedJSON
    )
  } catch (err) {
    console.error(err)
  }
}

run()
