const fs = require('fs')
const path = require('path')
const bindings = require('bindings')
const binding = bindings('snapdiff')

const FileType = {
  AUTO    : 0,
  JSON    : 1,
  GZIP    : 2
}
const FileTypeUPPER = FileType.GZIP
const FileTypeUNKNOWN = -1

function tryDetectFileType(fullPath) {
  const ext = path.extname(fullPath)
  switch (ext) {
    case '.gz':
      return FileType.GZIP
    case '.json':
    case '.heapsnapshot':
      return FileType.JSON
  }
  return FileTypeUNKNOWN
}

function processFile(fullPath, maxStringLength, fileType, debug, cb) {
  const result = binding.aggregate(fullPath, maxStringLength, fileType, debug)
  const errorMsg = result[0], json = result[1]
  if (errorMsg == null) return cb(null, json)
  return cb(new Error(errorMsg))
}

const defaultAggregateOpts = {
  maxStringLength: 50,
  fileType: FileType.AUTO,
  debug: false
}

function aggregate(opts, cb) {
  opts = Object.assign({}, defaultAggregateOpts, opts)
  const fullPath = opts.fullPath
  if (fullPath == null) {
    return setImmediate(() =>
      cb(new Error('Please supply a fullPath to the file containing the snapshot'))
    )
  }

  const fileType = opts.fileType === FileType.AUTO
    ? tryDetectFileType(fullPath)
    : opts.fileType

  if (fileType <= FileTypeUNKNOWN || fileType > FileTypeUPPER) {
    return setImmediate(() =>
      cb(
        new Error(
          `Unable to detect filetype of ${fullPath}, please supply FileType.JSON or FileType.GZIP`
      ))
    )
  }

  fs.access(fullPath, onaccessed)
  function onaccessed(err) {
    if (err) return cb(err)
    processFile(fullPath, opts.maxStringLength, fileType, !!opts.debug, cb)
  }
}

exports.aggregate = aggregate
exports.FileType = FileType
